#!/usr/bin/env python

from PyQt4 import QtGui
import sys
import os
import apt
import urllib
import stat
import trinity_installer_gui
import subprocess
from datetime import datetime

with open('trinity_packages.txt') as f:
    MyApps = f.read().splitlines()


class ExampleApp(QtGui.QMainWindow, trinity_installer_gui.Ui_MainWindow):

    def __init__(self, parent=None):
        # print datetime.now().time(), "init"
        super(ExampleApp, self).__init__(parent)
        self.setupUi(self)
        self.home()


    def home(self):
        # print datetime.now().time(), "home"
        self.cache = apt.Cache()
        self.tableAppList.itemSelectionChanged.connect(self.aboutApp)
        self.btnInstall.clicked.connect(self.markInstall)
        self.btnRemove.clicked.connect(self.markRemove)
        self.btnCommit.clicked.connect(self.btnCommit_pressed)

        self.btnInstall.hide()
        self.btnRemove.hide()
        self.btnCommit.hide()
        self.InfoSize.clear()
        self.populateApps()

    def btnCommit_pressed(self, btn):
        # print(datetime.now().time(), "Button Commit pressed")
        if os.geteuid() != 0:
            self.showuserdialog()
        else:
            self.cache.commit()
            self.cache.open(None)
            self.btnCommit.hide()
            self.populateApps()

    def markInstall(self, btn):
        # print datetime.now().time(), "markInstall"
        if os.geteuid() != 0:
            self.showuserdialog()
        else:
            thisName = self.tableAppList.selectedIndexes()[1].data().toString()
            self.pkg = self.cache[str(thisName)]
            self.pkg.mark_install()
            self.btnCommit.show()
            print("Install " + thisName)

    def markRemove(self, btn):
        # print datetime.now().time(), "markRemove"
        if os.geteuid() != 0:
            self.showuserdialog()
        else:
            thisName = self.tableAppList.selectedIndexes()[1].data().toString()
            self.pkg = self.cache[str(thisName)]
            self.pkg.mark_delete()
            self.btnCommit.show()
            print("Remove " + thisName)

    def get_full_install_size(self):
        # print datetime.now().time(), "get_full_install_size"
        try:
            appName = self.tableAppList.selectedIndexes()[1].data().toString()
            appStatus = self.tableAppList.selectedIndexes()[2].data().toString()
        except IndexError:
            pass
        try:
            pkg = self.cache[str(appName)]
            tot_size = 0
            itemSize = self.getVersion(appName).installed_size
            if appStatus == "Available":
                tot_size += itemSize
                for deplist in pkg.candidate.get_dependencies("Depends"):
                    for dep in deplist.or_dependencies:
                        is_inst = self.checkInstalled(dep.name)
                        if is_inst == False:
                            dep_size = self.getVersion(dep.name).installed_size
                            tot_size += dep_size
        except KeyError:
            pass
        return self.sizeHr(tot_size)

    def populateApps(self):
        # print datetime.now().time(), "populateApps"
        if self.tableAppList.rowCount() > 0:
            self.tableAppList.setRowCount(0)
        location2 = ":/images/trinity.png"
        myitem = QtGui.QTableWidgetItem()
        myicon = QtGui.QIcon()
        myicon.addPixmap(QtGui.QPixmap(location2), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        myitem.setIcon(myicon)
        for item in MyApps:
            install_status = self.checkInstalled(item)
            if install_status == "Ignore":
                self.add_row_to_table(myitem,item,"Not Available",0,0,0)
            elif install_status:
                mycategory, itemVersion, HrSize = self.getItemInfo(item)
                self.add_row_to_table(myitem,item, "Installed", str(itemVersion), "N/A", str(mycategory))
            else:
                mycategory, itemVersion, HrSize = self.getItemInfo(item)
                self.add_row_to_table(myitem, item, "Available", str(itemVersion), HrSize, str(mycategory))
        self.tableAppList.resizeColumnsToContents()

    def aboutApp(self):
        # print datetime.now().time(), "aboutApp"
        try:
            thisName = self.tableAppList.selectedIndexes()[1].data().toString()
            if self.tableAppList.selectedIndexes()[2].data().toString() == "Not Available":
                self.btnInstall.hide()
                self.btnRemove.hide()
                self.InfoBottom.clear()
                self.InfoSize.clear()
            else:
                self.showInfo(thisName)
        except IndexError:
            self.btnInstall.hide()
            self.btnRemove.hide()
            self.InfoSize.clear()
            self.InfoBottom.clear()

    def getItemInfo(self, item):
        # print datetime.now().time(), "getItemInfo"
        self.pkg = self.cache[str(item)]
        return self.pkg.section, self.pkg.versions[0], self.sizeHr(self.pkg.versions[0].installed_size)


    def getVersion(self, item):
        # print datetime.now().time(), "getVersion"
        self.pkg = self.cache[str(item)]
        versions = self.pkg.versions
        return versions[0]

    def checkInstalled(self, appName):
        # print datetime.now().time(), "checkInstalled", appName
        try:
            self.pkg = self.cache[appName]
            if self.pkg.is_installed:
                return True
            else:
                return False
        except KeyError:
            return "Ignore"

    def add_row_to_table(self, c1, c2, c3, c4, c5, c6):
        # print datetime.now().time(), "add_row_to_table"
        rowPosition = self.tableAppList.rowCount()
        self.tableAppList.insertRow(rowPosition)
        self.tableAppList.setItem(rowPosition , 0, QtGui.QTableWidgetItem(c1))
        self.tableAppList.setItem(rowPosition , 1, QtGui.QTableWidgetItem(c2))
        self.tableAppList.setItem(rowPosition , 2, QtGui.QTableWidgetItem(c3))
        self.tableAppList.setItem(rowPosition , 3, QtGui.QTableWidgetItem(c4))
        self.tableAppList.setItem(rowPosition , 4, QtGui.QTableWidgetItem(c5))
        self.tableAppList.setItem(rowPosition , 5, QtGui.QTableWidgetItem(c6))

    def showInfo(self, name):
        # print datetime.now().time(), "showInfo"
        self.InfoBottom.clear()
        pkg = self.cache[str(name)]
        versions = pkg.versions
        self.InfoBottom.append(versions[0].description)
        self.InfoBottom.moveCursor(QtGui.QTextCursor.Start)
        self.InfoSize.clear()
        apptotsize = self.get_full_install_size()
        if apptotsize == "0.0B":
            apptotsize = "Installed"
            self.btnRemove.show()
            self.btnInstall.hide()
        else:
            self.btnRemove.hide()
            self.btnInstall.show()
        self.InfoSize.setText("Approximate install size (with dependencies) ( " + apptotsize + " )")

    def sizeHr(self, num, suffix='B'):
        # print datetime.now().time(), "sizeHr"
        for unit in ['','Ki','Mi','Gi','Ti','Pi','Ei','Zi']:
            if abs(num) < 1024.0:
                return "%3.1f%s%s" % (num, unit, suffix)
            num /= 1024.0
        return "%.1f%s%s" % (num, 'Yi', suffix)

    def showuserdialog(self):
        QtGui.QMessageBox.about(self, "Trinity Package Installer", "You cannot do this as user\nPlease run the script as root" )

    def closeEvent(self, event):
        choice = QtGui.QMessageBox.question(self, 'Trinity Package Installer', 'Do you want to quit the applictaion?', QtGui.QMessageBox.Yes | QtGui.QMessageBox.No)
        if choice == QtGui.QMessageBox.Yes:
            event.accept()
        else:
            event.ignore()


def main():
    app = QtGui.QApplication(sys.argv)
    form = ExampleApp()
    form.show()
    app.exec_()


if __name__ == '__main__':
    main()
